"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Progress = void 0;
var STATE = {
    DISAPPEAR: 0,
    NOTHING: 1,
    APPEAR: 2,
    PENDING: 3,
    DISAPPEAR_RESTART: 4,
};
var PERSIST_TIME = 150;
var Progress = /** @class */ (function () {
    function Progress(options) {
        if (options === void 0) { options = {}; }
        this._el = document.createElement('div');
        this._state = STATE.NOTHING;
        this._opts = {
            maxWidth: '99.8%',
            height: '4px',
            duration: 60000,
            hideDuration: 400,
            zIndex: '9999',
            color: '#ff1a59',
            className: '',
            timing: 'cubic-bezier(0,1,0,1)',
            position: 'top',
            container: document.body,
        };
        this._rafId = null;
        this._timerId = null;
        this._promises = [];
        this._delayTimers = [];
        this.setOptions(options);
    }
    Progress.prototype.setOptions = function (newOptions) {
        var opts = assign(this._opts, newOptions);
        if (!isNaN(opts.maxWidth))
            opts.maxWidth += 'px';
        if (!isNaN(opts.height))
            opts.height += 'px';
        opts.zIndex = String(opts.zIndex);
        this._el.className = opts.className;
        this._css(__assign({ height: opts.height, background: opts.color, zIndex: opts.zIndex, position: '', left: '', top: '', bottom: '' }, {
            top: {
                position: 'fixed',
                top: '0',
                left: '0',
            },
            bottom: {
                position: 'fixed',
                bottom: '0',
                left: '0',
            },
        }[opts.position]));
    };
    Progress.prototype._css = function (style) {
        assign(this._el.style, style);
    };
    Object.defineProperty(Progress.prototype, "isInProgress", {
        get: function () {
            return this._state !== 0;
        },
        enumerable: false,
        configurable: true
    });
    Progress.prototype.start = function () {
        var _this = this;
        switch (this._state) {
            case STATE.APPEAR:
            case STATE.PENDING:
            case STATE.DISAPPEAR_RESTART:
                return;
            case STATE.DISAPPEAR:
                this._state = STATE.DISAPPEAR_RESTART;
                return;
        }
        this._state = STATE.APPEAR;
        var opts = this._opts;
        var transition = "width ".concat(opts.duration, "ms ").concat(opts.timing);
        this._css({
            width: '0',
            opacity: '1',
            transition: transition,
            webkitTransition: transition,
        });
        opts.container.appendChild(this._el);
        this._rafId = requestAnimationFrame(function () {
            _this._rafId = requestAnimationFrame(function () {
                _this._rafId = null;
                _this._state = STATE.PENDING;
                _this._css({ width: _this._opts.maxWidth });
            });
        });
    };
    Progress.prototype.end = function (immediately) {
        var _this = this;
        if (immediately === void 0) { immediately = false; }
        this._promises = [];
        this._delayTimers.splice(0).forEach(clearTimeout);
        switch (this._state) {
            case STATE.NOTHING:
                return;
            case STATE.APPEAR:
                this._state = STATE.NOTHING;
                cancelAnimationFrame(this._rafId);
                this._rafId = null;
                detach(this._el);
                return;
            case STATE.DISAPPEAR:
            case STATE.DISAPPEAR_RESTART:
                if (immediately) {
                    this._state = STATE.NOTHING;
                    clearTimeout(this._timerId);
                    this._timerId = null;
                    detach(this._el);
                }
                else {
                    this._state = STATE.DISAPPEAR;
                }
                return;
        }
        if (immediately) {
            this._state = STATE.NOTHING;
            detach(this._el);
            return;
        }
        this._state = STATE.DISAPPEAR;
        var opts = this._opts;
        var transition = "width 50ms, opacity ".concat(opts.hideDuration, "ms ").concat(PERSIST_TIME, "ms");
        this._css({
            width: '100%',
            opacity: '0',
            transition: transition,
            webkitTransition: transition,
        });
        this._timerId = setTimeout(function () {
            _this._timerId = null;
            var restart = _this._state === STATE.DISAPPEAR_RESTART;
            _this._state = STATE.NOTHING;
            detach(_this._el);
            if (restart) {
                _this.start();
            }
        }, opts.hideDuration + PERSIST_TIME);
    };
    Progress.prototype.promise = function (p, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.min, min = _d === void 0 ? 100 : _d;
        var timerId;
        var start = function () {
            if (min > 0) {
                p = Promise.all([p, new Promise(function (res) { return setTimeout(res, min); })]).then(function (_a) {
                    var v = _a[0];
                    return v;
                });
            }
            _this._promises.push(p);
            _this.start();
        };
        var cleanupTimer = function () {
            var timers = _this._delayTimers;
            timers.splice(timers.indexOf(timerId) >>> 0, 1);
            timerId = null;
        };
        if (delay > 0) {
            this._delayTimers.push((timerId = setTimeout(function () {
                cleanupTimer();
                start();
            }, delay)));
        }
        else {
            start();
        }
        var onFinally = function () {
            if (timerId) {
                clearTimeout(timerId);
                cleanupTimer();
                return;
            }
            var promises = _this._promises;
            var idx = promises.indexOf(p);
            if (~idx) {
                promises.splice(idx, 1);
                if (promises.length === 0)
                    _this.end();
            }
        };
        return p.then(function (val) { return (onFinally(), val); }, function (err) { return (onFinally(), Promise.reject(err)); });
    };
    return Progress;
}());
exports.Progress = Progress;
function assign(target, src) {
    for (var k in src) {
        if (Object.prototype.hasOwnProperty.call(src, k))
            target[k] = src[k];
    }
    return target;
}
function detach(el) {
    if (el.parentNode)
        el.parentNode.removeChild(el);
}
